
def TryFind(string: str, substrings: list, index: int):
    '''
    Returns -1 if none of the substrings could not be found
    '''

    res_i = -1
    # Loop through supstrings
    for substr in substrings:
        # Store string into temporary editable variable
        raw = string
        # Loop through index
        for _ in range(index+1):
            if raw.find(substr) != -1:
                res_i = raw.find(substr)
            # Replace substring to prevent reading the same substring twice
            raw = raw.replace(substr, "█", 1)

        if raw.find("█") != -1:
            return res_i, substr
    return res_i, substr

def ReplaceMultStr(string: str, substrings: list, new_substring: str):
    '''
    Replace multiple substrings of a string
    '''

    for substr in substrings:
        string = string.replace(substr, new_substring)
    return string

def RemoveBetweenChar(string: str, chars: list, n: int=-1):
    '''
    Remove every substring which is between the chars in the chars list
    n: -1 = remove all
    '''

    # Loop through chars
    for char in chars:
        # If n == remove all
        if n == -1:
            while string.count(char) > 0:
                pos_1 = string.find(char)
                pos_2 = string.find(char, pos_1 + 1)

                if (pos_1 == -1 ) | (pos_2 == -1):
                    return string
                string = string[0:pos_1] + string[pos_2+1:]
        # If n != remove all
        else:
            for _ in range(n):
                pos_1 = string.find(char)
                pos_2 = string.find(char, pos_1 + 1)

                if (pos_1 == -1 ) | (pos_2 == -1):
                    return string
                string = string[0:pos_1] + string[pos_2+1:]
                
    return string

def FindIndexReverse(string: str, char: str, offset: int=0):
    '''
    Get the distance to left
    '''

    i = 0
    res_char = ''
    while res_char != char:
        i += 1
        res_char = string[offset-i]
    return offset - i



from HTMLReader.Read import *
from HTMLReader.strUtils import *


class Element:
    def __init__(self, raw_tag: str, content: str):
        '''
        A HTML element converted into python class
        '''

        self.content = content

        # Get tag type (eg. 'div' or 'h1')
        self.type = raw_tag.split(' ')[0].replace('<', '')

        # Get prop fields (eg. 'src', 'class' or / and 'id')
        props_fields = RemoveBetweenChar(ReplaceMultStr(raw_tag, [f'<{self.type} ', '=', '>'], ''), ['"', "'"]).split()
        self.props = {}

        # Get props
        for field in props_fields:
            # Get start & end index of prop values
            prop_start_i = TryFind(raw_tag, [f"{field}='", f'{field}="'], 0)[0] + len(f'{field}="')
            prop_end_i = TryFind(raw_tag[prop_start_i:], ["'", '"'], 0)[0] + prop_start_i

            # Split all prop values into list
            self.props[field] = raw_tag[prop_start_i:prop_end_i].split()

    @staticmethod
    def GetWithTag(raw_html: str, tag_type: str):
        '''
        Get all elements with the given tag
        '''
        
        elements = []

        while raw_html.count(tag_type) > 0:

            # Calculate opening tag index & get opening tag content (eg. '<div id="sample-id" class="content">')
            opening_tag_start_i = raw_html.find(tag_type) - 1
            opening_tag_end_i = raw_html[opening_tag_start_i:].find('>') + opening_tag_start_i + 1
            opening_tag_content = raw_html[opening_tag_start_i:opening_tag_end_i]

            if raw_html[opening_tag_start_i] != '<':
                raw_html = raw_html.replace(tag_type, '█'*len(tag_type), 1)
                continue

            # Get closing tag index with GetClosingTag() function
            closing_tag_i = GetClosingTag(raw_html, opening_tag_content)
            
            # Get the content between the opening and closing tag
            content = raw_html[opening_tag_end_i:closing_tag_i]
            
            # Return new HTML element object
            elements.append(Element(opening_tag_content, content))

            raw_html = raw_html.replace(tag_type, '█'*len(tag_type), 1)

        return elements

    @staticmethod
    def Get(raw_html: str, identification: str):
        '''
        Get all elements with the given class
        '''
        
        elements = []

        while raw_html.count(identification) > 0:

            identification_i = raw_html.find(identification)

            # Calculate opening tag index & get opening tag content (eg. '<div id="sample-id" class="content">')
            opening_tag_start_i = FindIndexReverse(raw_html, '<', identification_i)
            opening_tag_end_i = raw_html[opening_tag_start_i:].find('>') + opening_tag_start_i + 1
            opening_tag_content = raw_html[opening_tag_start_i:opening_tag_end_i]

            # Get closing tag index with GetClosingTag() function
            closing_tag_i = GetClosingTag(raw_html, opening_tag_content)
            
            # Get the content between the opening and closing tag
            content = raw_html[opening_tag_end_i:closing_tag_i]
            
            # Return new HTML element object
            elements.append(Element(opening_tag_content, content))

            raw_html = raw_html.replace(identification, '█'*len(identification), 1)

        return elements



def GetClosingTag(raw_html: str, opening_tag: str):
    '''
    Get the belonging closing tag with the given opening tag
    '''
    
    opening_start_i = raw_html.find(opening_tag)
    opening_end_i = opening_start_i + len(opening_tag)

    # Get tag type (eg. 'div' or 'h1')
    tag_type = opening_tag.split(' ')[0].replace('<', '')

    # Cut the opening tag off the raw html
    raw_html = raw_html[opening_end_i:]
    # Get number of other similar opening tags
    num_tags = raw_html.count(f'<{tag_type}')

    # Loop through every other opening tag and find the related closing tag
    for _ in range(num_tags + 1):
        closing_start_i = raw_html.find(f'</{tag_type}')
        closest_opening_i = raw_html.find(f'<{tag_type}')

        if closest_opening_i > closing_start_i:
            break

        # Cut the everything off to this closing tag (including closing tag)
        raw_html = raw_html.replace(f'</{tag_type}>', '█'*len(f'</{tag_type}>'), 1)

    return closing_start_i + opening_end_i
